#ifndef FLY_SECRETSSERVICEJOBS_H
#define FLY_SECRETSSERVICEJOBS_H

#include <QExplicitlySharedDataPointer>
#include <QSharedPointer>

#include <flyjob.h>
#include <flycompositejob.h>
#include <flysecretsserviceitem.h>

class QDBusObjectPath;

namespace FlySecretsService
{

class SearchItemsJobPrivate;
class UnlockJobPrivate;

class FLY_SECRETSSERVICE_EXPORT ServiceJob : public FlyCompositeJob
{
	Q_OBJECT
	Q_DISABLE_COPY(ServiceJob)
public:
	explicit ServiceJob( QObject* parent = 0 );
	virtual ~ServiceJob();

	enum ServiceError
	{
		UndefinedError = -1,            /// this error should never be encountered
		NoError = 0,
		InternalError,
		OperationCancelledByTheUser,    /// the user choose to cancel ther operation during a message prompt
		MissingParameterError,
		PathsNotFound
	};

	/**
	 * This override is intended to make it public and as such let subjob additions from other classes in this client implementation
	 */
	virtual bool addSubjob(FlyJob* theJob); // override

protected:
	/**
	 * Request job abort on error
	 * @param err is the error code
	 * @param errTxt is the internationalized error message user might see
	 */
	void finishedWithError( ServiceError err, const QString& errTxt );

	/**
	 * Request job finish without error condition
	 */
	void finishedOk();
};

class FLY_SECRETSSERVICE_EXPORT SearchItemsJob : public ServiceJob
{
	Q_OBJECT
	Q_DISABLE_COPY(SearchItemsJob)
public:
	explicit SearchItemsJob( const QMap<QString, QString>& attributes, QObject* parent = 0 );
	virtual ~SearchItemsJob();

	typedef QExplicitlySharedDataPointer< SecretItem > Item;
	typedef QList< Item > ItemList;
	ItemList unlockedItems() const;
	ItemList lockedItems() const;
	virtual void start();

private:
	friend class SearchItemsJobPrivate;
	QSharedPointer<SearchItemsJobPrivate> d;
};

class FLY_SECRETSSERVICE_EXPORT UnlockJob : public ServiceJob
{
	Q_OBJECT
	Q_DISABLE_COPY(UnlockJob)
public:
	UnlockJob( const QList<QDBusObjectPath>& paths, const WId winId, QObject* parent = 0 );

	virtual void start();

	QList<QDBusObjectPath> unlockedPaths() const;

private:
	friend class UnlockJobPrivate;
	QSharedPointer< UnlockJobPrivate > d;
};

}

#endif
